package sample3;

import java.rmi.RemoteException; 
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.jms.*;
import javax.naming.*;

public class Sample3EJB implements SessionBean
{
 
  final private String CLIENTID = "JBK";
  final private String SUBNAME = "Sample3Sub";

  final private String TCFNAME = "jms/Samples/TCF1";
  final private String TOPICNAME = "jms/Samples/T1";

  /** Create a durable subscription to receive messages from the broker.
   */
  public void createSubscription()
    throws Exception
  {
    TopicConnectionFactory tcf = null;
    TopicConnection jmsCon = null;
    TopicSession session = null;

    System.out.println("Sample3EJB: createSubscription");

    try {
      // Lookup the topic connection factory from JNDI
      InitialContext ic = new InitialContext();
      tcf = (TopicConnectionFactory) ic.lookup(TCFNAME);

      if (tcf == null) {
        System.out.println("Topic connection factory not found!");
        throw new Exception("Topic connection factory not found!");
      }

      // Create a connection
      System.out.println("creating connection");
      jmsCon = tcf.createTopicConnection();
      jmsCon.start();

      // Create a session
      session = jmsCon.createTopicSession(true, Session.AUTO_ACKNOWLEDGE);

      System.out.println("looking up topic");
      Topic topic = (Topic) ic.lookup(TOPICNAME);
      
      // Get around the problem of publish before subscribe by
      // using a durable subscription.
      System.out.println("creating durable subscriber");
      TopicSubscriber sub = session.createDurableSubscriber(topic, SUBNAME);

      // Close the subscriber straight away - NB because it is durable the *subscription*
      // remains after the *subscriber* is closed.
      sub.close();

      System.out.println("closing connection");
      jmsCon.close();
      jmsCon = null;

      System.out.println("done");

    } catch (JMSException je) {
      System.out.println("failed with "+je);
      Exception le = je.getLinkedException();
      if (le != null) System.out.println("linked exception "+le);

      throw je;

    } catch (Exception e) {
      System.out.println("failed with "+e);
      throw e;

    } finally {
      // Ensure connection gets closed
      if (jmsCon != null) {
        try {
          jmsCon.close();
        } catch (JMSException je) {
        }
      }
    }
  }


  /** Delete the durable subscription.
   */
  public void destroySubscription()
    throws Exception
  {
    TopicConnectionFactory tcf = null;
    TopicConnection jmsCon = null;
    TopicSession session = null;

    System.out.println("Sample3EJB destroySubscription");

    try {

      System.out.println("Looking up Topic Connection Factory");
      InitialContext ic = new InitialContext();
      tcf = (TopicConnectionFactory) ic.lookup(TCFNAME);

      // Create the connection
      jmsCon = tcf.createTopicConnection();
      jmsCon.start();

      // Create a session
      System.out.println("Creating session");
      session = jmsCon.createTopicSession(true, Session.AUTO_ACKNOWLEDGE);

      // Delete the durable subscription
      System.out.println("deleting subscription");
      session.unsubscribe(SUBNAME);

      System.out.println("closing connection");
      jmsCon.close();
      jmsCon = null;

      System.out.println("done");

    } catch (JMSException je) {
      System.out.println("failed with "+je);
      Exception le = je.getLinkedException();
      if (le != null) System.out.println("linked exception: "+le);
      throw je;

    } catch (Exception e) {
      System.out.println("failed with "+e);
      throw e;

    } finally {
      // Ensure connection gets closed
      if (jmsCon != null) {
        try {
          jmsCon.close();
        } catch (JMSException je) {
        }
      }
    }

  }


  /** Publish message.
   */
  public String putMessage(String text)
    throws Exception
  {
    TopicConnectionFactory tcf = null;
    TopicConnection jmsCon = null;
    TopicSession session = null;
    TopicPublisher pub = null;
    String mID = null;

    System.out.println("Sample3EJB: request to put message '"+text+"'");

    try {
      // Lookup the topic connection factory
      InitialContext ic = new InitialContext();
      tcf = (TopicConnectionFactory) ic.lookup(TCFNAME);

      if (tcf == null) {
        System.out.println("Topic connection factory not found!");
        throw new Exception("Topic connection factory not found!");
      }

      // Create a connection
      System.out.println("creating connection");
      jmsCon = tcf.createTopicConnection();
      jmsCon.start();

      // Create a session
      System.out.println("Creating session");
      session = jmsCon.createTopicSession(true, Session.AUTO_ACKNOWLEDGE);

      // Lookup topic
      Topic topic = (Topic) ic.lookup(TOPICNAME);

      // Create a publisher
      System.out.println("creating topic publisher");
      pub = session.createPublisher(topic);
      
      // Create a message to send...
      TextMessage message = session.createTextMessage(text);

      // ...and send it
      System.out.println("sending message");
      pub.publish(message);

      mID =  message.getJMSMessageID();

      // tidy up
      pub.close();
      session.close();
      jmsCon.close();
      jmsCon = null;

      System.out.println("done");

    } catch (JMSException je) {
      System.out.println("failed with "+je);
      Exception le = je.getLinkedException();
      if (le != null) System.out.println("linked exception "+le);
      throw je;

    } catch (Exception e) {
      System.out.println("failed with "+e);
      throw e;

    } finally {
      // Ensure connection gets closed
      if (jmsCon != null) {
        try {
          jmsCon.close();
        } catch (JMSException je) {
        }
      }
    }

    return mID;
  }


  /** Retrieve a message from the durable subscription.
   */
  public String getMessage()
    throws Exception
  {
    java.sql.Connection con;
    TopicConnectionFactory tcf = null;
    TopicConnection jmsCon = null;
    TopicSession session = null;

    String body = null;

    System.out.println("Sample3.getMessage()");

    try {
      // Lookup topic connection factory
      InitialContext ic = new InitialContext();
      System.out.println("looking up Topic Connection Factory");
      tcf = (TopicConnectionFactory) ic.lookup(TCFNAME);

      // Create connection
      jmsCon = tcf.createTopicConnection();
      jmsCon.start();

      // Create a session
      System.out.println("Creating session");
      session = jmsCon.createTopicSession(true, Session.AUTO_ACKNOWLEDGE);

      System.out.println("looking up topic");
      Topic topic = (Topic) ic.lookup(TOPICNAME);

      // Create a subscriber against the existing durable subscription
      System.out.println("creating subscriber");
      TopicSubscriber sub = session.createDurableSubscriber(topic, SUBNAME);

      // Get a message
      System.out.println("subscribe");
      Message inMessage = sub.receive(5000);

      if (inMessage == null) {
        System.out.println("no message received");

      } else if (inMessage instanceof TextMessage) {
        body = ((TextMessage)inMessage).getText();
        System.out.println("body is >"+body+"<");

      } else if (inMessage instanceof BytesMessage) {
        body = ((BytesMessage)inMessage).readUTF();
        System.out.println("body is >"+body+"<");

      } else {
        System.out.println("message not text or bytes");
      }

      System.out.println("closing subscriber");
      sub.close();

      System.out.println("closing connection");
      jmsCon.close();
      jmsCon = null;

    } catch (JMSException je) {
      System.out.println("failed with "+je);
      Exception le = je.getLinkedException();
      if (le != null) System.out.println("linked exception: "+le);
      throw je;

    } catch (Exception e) {
      System.out.println("failed with "+e);
      throw e;

    } finally {
      // Ensure connection gets closed
      if (jmsCon != null) {
        try {
          jmsCon.close();
        } catch (JMSException je) {
        }
      }
    }

    return body;

  }


  public void ejbCreate()
  {
    System.out.println("Sample3.ejbCreate()");
  }

  public void ejbRemove()
  {
    System.out.println("Sample3.ejbRemove()");
  }

  public void ejbActivate()
  {
    System.out.println("Sample3.ejbActivate()");
  }

  public void ejbPassivate()
  {
    System.out.println("Sample3.ejbPassivate()");
  }

  public void setSessionContext(SessionContext sc) {}

}
  

